
#ifndef _UARTCAN_H_
#define _UARTCAN_H_

#include <windows.h>
#include <stdio.h>
#include <mmsystem.h>


#define DEFAULT_TIMEOUT				1000
#define FRAME_SIZE					12800


typedef enum
{
	CAN_MODE_20AB	= 0,
	CAN_MODE_20A	= 1,
	CAN_MODE_20B	= 2
}ENUM_CAN_MODE;

typedef enum
{
	CAN_IDTYPE_20A	= 0,
	CAN_IDTYPE_20B	= 1
}ENUM_CAN_ID_TYPE;

typedef enum
{
	CAN_DATA_FRAME		= 0,
	CAN_REMOTE_FRAME	= 1
}ENUM_CAN_DATA_TYPE;

typedef enum
{
	RECEIVE_OK	= 0,
	CAN_ERROR	= 1,
	UART_ERROR	= 2
}ENUM_RECV;

typedef enum
{
	TX_WARNING			= 0,
	RX_WARNING			= 1,
	TX_ERROR_PASSIVE	= 2,
	RX_ERROR_PASSIVE	= 3,
	BUSOFF_ERROR		= 4
}ENUM_ERROR;

typedef enum
{
	READ_RECV_ENV	= 0,
	STOP_RECV_ENV	= 1,
	START_RECV_ENV	= 2
}ENUM_RECV_ENV;

typedef struct
{
	BYTE	MessageMode;		// 0 : CAN2.0A/B, 1 : CAN2.0A,  2 : CAN2.0B
	BOOL	IDMaskType;			// MessageMode 0̰  Ʈ 0ΰ : ID, MASK 2.0A
								// MessageMode 0̰  Ʈ 1ΰ : ID, MASK 2.0B
								// MessageMode 1  :  Ʈ  ID,MASK 2.0A
								// MessageMode 2  :  Ʈ  ID,MASK 2.0B
	BOOL	BusOFFtoReset;		// 1 : CAN Bus-Off߻  ڵ CAN Reset 
	BOOL	BusOFFtoError;		// 1 : CAN Bus-Off߻    ˸
	BOOL	PassiveError;		// 1 : CAN , Erorr-Passive߻   ˸
	BOOL	WarningError;		// 1 : CAN , Warning߻   ˸
	DWORD	BitTime;			// Bit Time
	DWORD	ID;					//  ID
	DWORD	Mask;				//  Mask
}STRUCT_CAN_ENV;

typedef struct
{
	BOOL	MessageMode;		// 0 : CAN2.0A, 1 : CAN2.0B
	BOOL	MessageType;		// 0 : Data Frame, 1 : Remote Frame
	int		DataSize;			//   : 0 ~ 8
	DWORD	ID;					// ID
	BYTE	Data[8];			// Data
}STRUCT_CAN_DATA;

class _declspec(dllexport) CUartCAN
{
public:
	CUartCAN();

	// COMƮ ʱȭ մϴ.
	BOOL			ComPortInit( int nCom, int nBPS );

	// COMƮ ִ Ȯϴ ԼԴϴ.
	BOOL			ComPortIsOpened( void );

	// COMƮ ݽϴ.
	void			ComPortClose( void );

	// COMƮ ۸ ʱȭմϴ.
	void			ComPortPurge( void );

	// COMƮ ͸ ϴ.
	BOOL			SendData(char *szData);

	// COMƮ  ͸ нϴ.	
	BOOL			ReadData(char *szData, int nTimeout);

	// ͼ   ޽ մϴ.
	void			SetMessage(HWND *pParent, UINT nMessage);

	// ͼ  ͸  ۸ մϴ.
	void			SetBuffer(STRUCT_CAN_DATA *stRecvData);



	// UART_CAN ȯ漳  ԼԴϴ.
	BOOL			WriteEnv(STRUCT_CAN_ENV *stData);

	// UART_CAN ȯ漳 о ԼԴϴ.
	BOOL			ReadEnv(STRUCT_CAN_ENV *stData);

	// UART_CAN CAN͸ ۽ϴ ռԴϴ.
	BOOL			WriteData(STRUCT_CAN_DATA *stData);

	// UART_CAN Ÿ带 ϴ ԼԴϴ.
	BOOL			RecvEnv(int nCommand, BOOL *bStatus);

	// UART_CAN մϴ.
	BOOL			Reset();
	
	// UART_CAN ߿  оɴϴ.
	BOOL			GetVersion(int *nMajor, int *nMinor);


protected:

	HANDLE			m_hComm;

	OVERLAPPED		m_ovWrite;
	OVERLAPPED		m_ovRead;
	COMMTIMEOUTS	m_ct;
	DCB				m_dcb;

	HANDLE			m_hThread;
	char			m_szThreadRxBuff[FRAME_SIZE];
	int				m_nThreadRxBuffLen;
	char			m_szThreadTxBuff[FRAME_SIZE];
	int				m_nThreadTxBuffLen;

	BOOL			m_bShutdownEvent;
	BOOL			m_bWriteEvent;

	HWND*			m_pParent;
	UINT			m_Message;

	BOOL			m_bRecvResponse;

	STRUCT_CAN_DATA	*m_stRecvData;

	BOOL			MakeThread();
	BOOL			DestroyThread();
	static UINT		COMThread(LPVOID pParam);

	DWORD			WriteToPort( BYTE *pData, DWORD nSize );
	DWORD			ReadFromPort( BYTE *pData );

	void			RecvData();

	int				MakeCheckSum(char* szData);

};


#endif /* _UARTCAN_H_ */